@@ -1,6 +1,6 @@
 import os
 
-__version__ = "0.4.0"
+__version__ = "0.4.1"
 __gitversion__ = __version__
 
 basePath = os.path.abspath(os.path.dirname(__file__))
@@ -8,7 +8,7 @@ basePath = os.path.abspath(os.path.dirname(__file__))
 try:
     from subprocess import check_output
     cmd = "--git-dir={}/../.git".format(basePath)
-    res = check_output(["git", cmd, "rev-parse", "--short", "HEAD"])[:-1]
+    res = check_output(["git", cmd, "rev-parse", "--short", "HEAD"], stdout=None)[:-1]
     __gitversion__ = "{} ({})".format(__version__, res)
 except Exception, e: # pragma: no cover
     pass
