@@ -1,4 +1,7 @@
 from twisted.trial.unittest import TestCase
+from twisted.python.modules import getModule
+
+from shutil import copy
 
 from saratoga.api import SaratogaAPI
 
@@ -7,20 +10,27 @@ import json
 class APIImpl(object):
     class v1(object):
         def example_GET(self, request, params):
-            return params
+            return params["params"]
 
         def listResponse_GET(self, request, params):
-            return params["data"]
+            return params["params"]["data"]
+
+        def exception_GET(self, request, params):
+            raise Exception("OMG LOL WTF")
+
+        def exampleregex_GET(self, request, params, someID):
+            return {"id": someID}
 
         def __init__(self):
             self.jsonbodyParams_GET = self.example_GET
             self.urlParams_GET = self.example_GET
             self.requestParams_GET = self.example_GET
             self.responseParams_GET = self.example_GET
+            self.responseParamsExtLoad_GET = self.example_GET
             self.dictResponse_GET = self.listResponse_GET
-            self.listofdictResponse_GET = self.listResponse_GET
-            self.paramOptions_GET = self.example_GET
             self.requiresAuth_GET = self.example_GET
+            self.invalidParamsType_GET = self.example_GET
+            self.invalidResponseFormat_GET = self.example_GET
 
 
 APIDef = {
@@ -31,50 +41,75 @@ APIDef = {
             "getProcessors": [{"versions": [1]}]
         },
         {
-            "endpoint": "jsonbodyParams",
+            "func": "exampleregex",
+            "endpoint": r"example/(\d+)",
             "getProcessors": [{"versions": [1]}]
         },
-        {
-            "endpoint": "urlParams",
-            "getProcessors": [{"versions": [1], "paramsType": "url"}]
-        },
         {
             "endpoint": "requestParams",
             "getProcessors": [{
                 "versions": [1],
-                "requiredParams": ["hello", "goodbye"],
-                "optionalParams": ["the"]
+                "requestSchema": {
+                    "properties": {
+                        "hello": {},
+                        "goodbye": {},
+                        "the": {}
+                    },
+                    "additionalProperties": False,
+                    "required": ["hello", "goodbye"]
+                },
+            }]
+        },
+        {
+            "endpoint": "responseParamsExtLoad",
+            "getProcessors": [{
+                "versions": [1],
+                "responseSchema": "jsonschemaext.json"
             }]
         },
         {
             "endpoint": "responseParams",
             "getProcessors": [{
                 "versions": [1],
-                "requiredResponseParams": ["cake", "muffin"],
-                "optionalResponseParams": ["pizza"]
+                "responseSchema": {
+                    "properties": {
+                        "cake": {},
+                        "muffin": {},
+                        "pizza": {}
+                    },
+                    "required": ["cake", "muffin"],
+                    "additionalProperties": False
+                }
             }]
         },
         {
             "endpoint": "listResponse",
-            "getProcessors": [{"versions": [1], "responseFormat": "list"}]
-        },
-        {
-            "endpoint": "listofdictResponse",
-            "getProcessors": [{"versions": [1], "responseFormat": "listofdict"}]
+            "getProcessors": [{"versions": [1], "responseSchema": {
+                "type": "array"
+            }}]
         },
         {
             "endpoint": "dictResponse",
-            "getProcessors": [{"versions": [1], "responseFormat": "dict"}]
-        },
-        {
-            "endpoint": "paramOptions",
-            "getProcessors": [{"versions": [1], "requiredParams": [
-            {"param": "foo", "paramOptions": ["bar", {"data": "baz"}]}]}]
+            "getProcessors": [{"versions": [1], "responseSchema": {
+                "type": "object"
+            }}]
         },
         {
             "endpoint": "requiresAuth",
             "requiresAuthentication": True,
             "getProcessors": [{"versions": [1]}]
+        },
+        {
+            "endpoint": "exception",
+            "getProcessors": [{"versions": [1]}]
+        },
+        {
+            "endpoint": "invalidParamsType",
+            "getProcessors": [{"versions": [1], "paramsType":"bar"}]
+        },
+        {
+            "endpoint": "invalidResponseFormat",
+            "getProcessors": [{"versions": [1], "responseFormat":"bar"}]
         }
     ]
 }
@@ -160,6 +195,9 @@ class SaratogaErrorCatchingTests(TestCase):
 class SaratogaAPITests(TestCase):
 
     def setUp(self):
+        fp = getModule(__name__).filePath
+        copy(fp.parent().child("jsonschemaext.json").path, "jsonschemaext.json")
+        
         self.api = SaratogaAPI(APIImpl, APIDef)
 
     def test_getResource(self):
@@ -168,7 +206,6 @@ class SaratogaAPITests(TestCase):
         """
         self.assertIs(self.api.resource, self.api.getResource())
 
-
     def test_basic(self):
         """
         Basic Saratoga test.
@@ -181,6 +218,46 @@ class SaratogaAPITests(TestCase):
 
         return self.api.test("/v1/example").addCallback(rendered)
 
+    def test_basicWithEmptyParams(self):
+        """
+        Basic Saratoga test.
+        """
+        def rendered(request):
+            self.assertEqual(
+                json.loads(request.getWrittenData()),
+                {"status": "success", "data": {}}
+            )
+
+        return self.api.test("/v1/example", replaceEmptyWithEmptyDict=True
+            ).addCallback(rendered)
+
+    def test_basicRegex(self):
+        """
+        Basic Saratoga test, testing the regex stuff.
+        """
+        def rendered(request):
+            self.assertEqual(
+                json.loads(request.getWrittenData()),
+                {"status": "success", "data": {"id": "4"}}
+            )
+
+        return self.api.test("/v1/example/4").addCallback(rendered)
+        
+        
+    def test_handlingOfExceptions(self):
+        """
+        Test that throwing a generic exception is handled gracefully.
+        """
+        def rendered(request):
+            self.assertEqual(
+                json.loads(request.getWrittenData()),
+                {"status": "error", "data": "Internal server error."}
+            )
+            warnings = self.flushLoggedErrors()
+            self.assertEqual(warnings[0].getErrorMessage(), "OMG LOL WTF")
+
+        return self.api.test("/v1/exception").addCallback(rendered)
+
 
     def test_authRequiredWhenDefaultServiceClass(self):
         """
@@ -199,49 +276,6 @@ class SaratogaAPITests(TestCase):
         return self.api.test("/v1/requiresAuth").addCallback(rendered)
 
 
-    def test_paramOptions(self):
-        """
-        Test that param options work.
-        """
-        def rendered(request):
-            self.assertEqual(
-                json.loads(request.getWrittenData()),
-                {"status": "success", "data": {"foo": "bar"}}
-            )
-
-        return self.api.test("/v1/paramOptions", params={"foo": "bar"}
-            ).addCallback(rendered)
-
-
-    def test_paramOptionsFailure(self):
-        """
-        Test that it handles paramOptions failing gracefully.
-        """
-        def rendered(request):
-            self.assertEqual(
-                json.loads(request.getWrittenData()),
-                {"status": "fail",
-                "data": "'cake' isn't part of [\"bar\", \"baz\"] in foo"}
-            )
-
-        return self.api.test("/v1/paramOptions", params={"foo": "cake"}
-            ).addCallback(rendered)
-
-    def test_reservedParamsFailure(self):
-        """
-        Test that it handles responding with a reserved parameter gracefully.
-        """
-        def rendered(request):
-            self.assertEqual(
-                json.loads(request.getWrittenData()),
-                {"status": "fail",
-                "data": "Forbidden keyword."}
-            )
-
-        d = self.api.test("/v1/example", params={"saratoga_user": "aaa"})
-        return d.addCallback(rendered)
-
-
     def test_dictResponse(self):
         """
         Test that it allows a dict response.
@@ -255,6 +289,19 @@ class SaratogaAPITests(TestCase):
         return self.api.test("/v1/dictResponse",
             params={"data": {"hi": "there"}}).addCallback(rendered)
 
+    def test_dictResponsePOSTArgs(self):
+        """
+        Test that it allows a dict response.
+        """
+        def rendered(request):
+            self.assertEqual(
+                json.loads(request.getWrittenData()),
+                {"status": "success", "data": {"hi": "there"}}
+            )
+
+        return self.api.test("/v1/dictResponse",
+                             params={"data": {"hi": "there"}}, useBody=False).addCallback(rendered)
+
 
     def test_listResponse(self):
         """
@@ -269,20 +316,18 @@ class SaratogaAPITests(TestCase):
         return self.api.test("/v1/listResponse",
             params={"data": ["hi", "there"]}).addCallback(rendered)
 
-
-    def test_listofdictResponse(self):
+    def test_listResponsePOSTArgs(self):
         """
-        Test that it allows a list of dict response.
+        Test that it allows a list response.
         """
         def rendered(request):
             self.assertEqual(
                 json.loads(request.getWrittenData()),
-                {"status": "success", "data": [{"hi": "there"}]}
+                {"status": "success", "data": ["hi", "there"]}
             )
 
-        return self.api.test("/v1/listofdictResponse",
-            params={"data": [{"hi": "there"}]}).addCallback(rendered)
-
+        return self.api.test("/v1/listResponse",
+                             params={"data": [["hi", "there"]]}, useBody=False).addCallback(rendered)
 
     def test_dictResponseFailure(self):
         """
@@ -297,7 +342,7 @@ class SaratogaAPITests(TestCase):
             )
             warnings = self.flushLoggedErrors()
             self.assertEqual(warnings[0].getErrorMessage(),
-                "Result is not a dict.")
+                "[u'hi', u'there'] is not of type 'object'")
 
         d = self.api.test("/v1/dictResponse",
             params={"data": ["hi", "there"]})
@@ -317,7 +362,7 @@ class SaratogaAPITests(TestCase):
             )
             warnings = self.flushLoggedErrors()
             self.assertEqual(warnings[0].getErrorMessage(),
-                "Result is not a list.")
+                "{u'hi': u'there'} is not of type 'array'")
 
         d = self.api.test("/v1/listResponse",
             params={"data": {"hi": "there"}})
@@ -371,10 +416,10 @@ class SaratogaAPITests(TestCase):
         return d.addCallback(rendered)
 
 
-    def test_requiredResponseParamsReturnsErrorIfGivenExtra(self):
+    def test_extLoadJSONSchema(self):
         """
-        Test that required response params work, and will return an error if
-        given extras.
+        Test that it loads external JSON Schema by loading in something that it
+        will fail.
         """
         def rendered(request):
             self.assertEqual(
@@ -383,12 +428,10 @@ class SaratogaAPITests(TestCase):
                 "data": "Internal server error."}
             )
             warnings = self.flushLoggedErrors()
-            self.assertEqual(warnings[0].getErrorMessage(),
-                "Unexpected response parameters: 'foo'")
-
-        d = self.api.test("/v1/responseParams",
-            params={"cake": "yes", "muffin": "yes", "foo": "bar"})
-        return d.addCallback(rendered)
+            self.assertEqual(warnings[0].getErrorMessage(), "u'cake' is a "
+                "required property, u'muffin' is a required property")
+            
+        return self.api.test("/v1/responseParamsExtLoad").addCallback(rendered)
 
 
     def test_requiredResponseParamsReturnsErrorIfNotGiven(self):
@@ -403,8 +446,8 @@ class SaratogaAPITests(TestCase):
                 "data": "Internal server error."}
             )
             warnings = self.flushLoggedErrors()
-            self.assertEqual(warnings[0].getErrorMessage(),
-                "Missing response parameters: 'cake', 'muffin'")
+            self.assertEqual(warnings[0].getErrorMessage(), "'cake' is a "
+                "required property, 'muffin' is a required property")
 
         return self.api.test("/v1/responseParams").addCallback(rendered)
 
@@ -417,7 +460,8 @@ class SaratogaAPITests(TestCase):
             self.assertEqual(
                 json.loads(request.getWrittenData()),
                 {"status": "fail",
-                "data": "Missing request parameters: 'goodbye', 'hello'"}
+                "data": "'hello' is a required property, 'goodbye' is a "
+                    "required property"}
             )
 
         d = self.api.test("/v1/requestParams")
@@ -433,7 +477,8 @@ class SaratogaAPITests(TestCase):
             self.assertEqual(
                 json.loads(request.getWrittenData()),
                 {"status": "fail",
-                "data": "Unexpected request parameters: 'unspecified'"}
+                "data": "Additional properties are not allowed (u'unspecified'"
+                    " was unexpected)"}
             )
 
         d = self.api.test("/v1/requestParams", params={
@@ -458,32 +503,3 @@ class SaratogaAPITests(TestCase):
             "hello": "yes", "goodbye": "no", "the": "beatles"
             })
         return d.addCallback(rendered)
-
-
-    def test_jsonbodyParams(self):
-        """
-        Test that params delivered through JSON work.
-        """
-        def rendered(request):
-            self.assertEqual(
-                json.loads(request.getWrittenData()),
-                {"status": "success", "data": {"hello": "there"}}
-            )
-
-        d = self.api.test("/v1/jsonbodyParams", params={"hello": "there"})
-        return d.addCallback(rendered)
-
-
-    def test_urlParams(self):
-        """
-        Test that params delivered through URL params work.
-        """
-        def rendered(request):
-            self.assertEqual(
-                json.loads(request.getWrittenData()),
-                {"status": "success", "data": {"hello": "there"}}
-            )
-
-        d = self.api.test("/v1/urlParams", params={"hello": "there"},
-            useBody=False)
-        return d.addCallback(rendered)
